//////////////////////////////////////////////////////////////////////////////
//
//  Version.h
//  Win32::Daemon Perl extension version header file
//
//  Copyright (c) 1998-2008 Dave Roth
//  Courtesy of Roth Consulting
//  http://www.roth.net/
//
//  This file may be copied or modified only under the terms of either 
//  the Artistic License or the GNU General Public License, which may 
//  be found in the Perl 5.0 source kit.
//
//  2008.03.24  :Date
//  20080324    :Version
//////////////////////////////////////////////////////////////////////////////

#ifndef _VERSION_H_
    #define _VERSION_H_

    #define VERSION_STRING          "20080324"
    #define VERSION_HI              ( 0x7d8 )
    #define VERSION_LO              ( 0x144 )
    #define PROD_VERSION            0x7d8, 0x03, 0x18, 0x00
    #define FILE_VERSION            0x7d8, 0x03, 0x18, 0x00

#endif 

